#!/usr/bin/python

import sys
import threading
import socket
import math
import subprocess

pluginVersion = "1.0"

# unit type tags
tagInternetStatus		= "xt.airport.internetStatus"
tagAirportStatus		= "xt.airport.status"
tagAirportClient		= "xt.airport.client"

# menu handlers
keyRebootMenu			= "rebootAirport"
keyRebootScript			= "reboot"
keyScanNowScript		= "scanNow"

# interface keys
# keys to get the user settings from the edit interface dialog
keyScanFrequency		= "xt.airport.scantime" # how often in minutes to scan
keyIsRouter				= "xt.airport.isrouter" # true for the airport that is the router so we can pull that address into the IP address unit
keyScanClients			= "xt.airport.scanclients" # if true then create units for each client and turn them on and off

keyAirportAtIndex		 = "xt.airport.atindex"


# status keys
# change these if your language doesn't have the same words used in the 
# interface that we are pulling things from
#
keyConnected			= "Connected"

# the string that they set the image name to when all is OK
keyWorkingNormally		= "working normally"


listForNewUnits			= "Airport Info"

# addresses
# the internet status units have constant addresses
addressStatus			= "INTERNET"
addressIp				= "IP"

# name of the script handler in the ip address unit to call when there is a new IP address

IPChangedScriptHandler	= "InternetIPChanged"


# if this is different than the new one then we will send a change to XTension
currentInternetStatus	= ""
currentIPAddress		= ""
airportCount			= 0

scanThread 				= None
scanTimer 				= None
scanInterval			= 5
rebootThread			= None
airportHasError			= False # if any airport is in an error state we don't close
								# the airport utility and we scan every minute until it's online

controlLock				= threading.Lock() # used to make sure we don't overlap scans or reboot commands


#
# load XTension includes
#


from os.path import expanduser
userHomePath = expanduser( '~')
del expanduser

pluginLocationFile = open( userHomePath + '/Documents/XTension Support/.python_include_path.txt', 'r')
if pluginLocationFile.mode == 'r':
	pluginLocationPath = pluginLocationFile.read()
	pluginLocationFile.close
else:
	print( "Error opening the XTension Plugin Includes path file, cannot continue")
	sys.exit()

pluginLocationPath += "/current"
	
sys.path.append( pluginLocationPath )

from xtension_plugin.xtension_constants import *
from xtension_plugin.xtension_plugin import *

scriptsFolder = sys.path[0] + "/scripts/"


#
# Main Handlers start here
#



#
#	S T A R T   S C A N 
#
# called regularly by the scan timer
# forks a thread to do the actual work so that we do not fail to respond to a ping
# from XTension while we're waiting on the scripts
#

def startScan():
	
		
		scanThread = Thread( target=threadedScan, args=())
		scanThread.start()


#
# 	T H R E A D E D   S C A N 
#
# run in a thread started by the startScan method
# this does the actual work of the scan
def threadedScan():

	#
	# first try to acquire the lock non-blocking so we can log if we are called
	# while the lock is still on. For debugging problems with releasing it in case of errors
	#
	
	if not controlLock.acquire( False):
		XTension.writeLog( "Another control thread has already locked this resource, blocking the scan thread and waiting", xtLogRed)
		controlLock.acquire()

	#controlLock.acquire()

	if XTension.debugMode:
		XTension.writeLog( "begin airport scan")
		
	# all the methods in order that we need to call and check for error
	
	scanMethods = [ verifyAirportUtility, 
		getInternetStatus, 
		getCount,
		getStatus]
		
		
	for thisHandler in scanMethods:
		if not thisHandler():
			XTension.writeLog( "airport scan aborted", xtLogRed)
			quitAirportUtility()
			
			controlLock.release()			
			return
	
	# check for the airport error flag and make sure we are running the period for the
	# scan timer. Also don't quit the airport utility if there is an error
	
	if airportHasError:
		if scanTimer.period != 30:
			scanTimer.setPeriod( 30)
		
	else:
		requestedInterval = 60 * scanInterval
		
		if scanTimer.period != requestedInterval:
			scanTimer.setPeriod( requestedInterval)
			
		# only quit if there is no error
		quitAirportUtility()
		

	
	
	# update the run state to OK if we get this far won't send any messages if it was already OK
	# the individual calls above will set it to false if there was any error
	XTension.setRunState( xtRunStateOK, "")
	controlLock.release()	

#
#  F I L T E R   N A M E   T O   A D D R E S S
#
# client names often change or re-arrange as they are resolved, try to make them similar
# and take off the .local because it doesn't always show that. Addresses in XTension are always
# uppercase to make sure to set that too.

def filterNameToAddress( rawName):

	rawName = rawName.upper()

	if rawName.endswith( ".LOCAL"):
		rawName = rawName[:-6]

	# additionally sometimes the name has dashes instead of spaces and sometimes it has 's and other times not

	rawName = rawName.replace( " ", "-")
	rawName = rawName.replace( "'", "")
	
	return rawName

	
#
#	I N T E R N E T   S T A T U S
#
# get the internet status from the first image
# the script will return the status from the popover
# window which should be "connected" we might have to let the user configure this
# for other languages
#

def getInternetStatus():
	if XTension.debugMode:
		XTension.writeLog( "begin getInternetStatus")
		
	try:
		reply = subprocess.check_output( ["osascript", scriptsFolder + "get_internet_status.scpt", "-s", "o"])
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no message"
		
		XTension.writeLog( "Error getting internet status (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		XTension.setRunState( xtRunStateError, "error getting internet status (" + str( e.returncode) + ") "  + theMessage )
		return False
		
	reply = reply.strip()
		
	# if we get the timeout message then also return an error
	if reply == 'timeout':
		XTension.writeLog( "Timeout while getting internet status", xtLogRed)
		XTension.setRunState( xtRunStateError, "timeout while getting internet status")
		return False
	
	
	# if we haven't already created the internet status unit in XTension then we need to do so
	
	if XTension.getUnitFromAddress( tagInternetStatus, "INTERNET") == None:
		x = XTCommand()
		x.set( xtKeyCommand, xtCommandNewUnit)
		x.set( xtKeyTag, tagInternetStatus)
		x.set( xtKeyAddress, "INTERNET")
		x.set( xtKeyName, "Internet Status: " + XTension.interfaceName )
		x.set( xtKeyList, listForNewUnits)
		x.set( xtKeyReadOnly, xtTrue)
		
		XTension.sendCommand( x)
		
	if XTension.getUnitFromAddress( tagInternetStatus, "IP") == None:
		x = XTCommand()
		x.set( xtKeyCommand, xtCommandNewUnit)
		x.set( xtKeyTag, tagInternetStatus)
		x.set( xtKeyAddress, "IP")
		x.set( xtKeyName, "IP Address: " + XTension.interfaceName)
		x.set( xtKeyList, listForNewUnits)
		x.set( xtKeyReadOnly, xtTrue)
		x.set( xtKeyDefaultLabel, "IP")
		
		XTension.sendCommand( x)
	
	# only send any more info if the value has changed from the current value

	global currentInternetStatus	
	if reply != currentInternetStatus:
	
		currentInternetStatus = reply
		
		x = XTCommand()
		x.set( xtKeyAddress, addressStatus) # only 1 internet address
		x.set( xtKeyTag, tagInternetStatus)
		x.set( xtKeyDefaultLabel, reply)		# also send whatever the text is as the default label
		x.set( xtKeyUpdateOnly, xtTrue)

		
		if reply == keyConnected:
			# we are connected
			x.set( xtKeyCommand, xtCommandOn)
			x.set( xtKeyDefaultLabel, "Up")
			if XTension.debugMode:
				XTension.writeLog("internet is up: (" + reply + ")")

		else:
			x.set( xtKeyCommand, xtCommandOff)
			x.set( xtKeyDefaultLabel, "Down")
			if XTension.debugMode:
				XTension.writeLog("internet is down: (" + reply + ")")


		XTension.sendCommand( x)
		if XTension.debugMode:
			XTension.writeLog("reply was: (" + reply + ")")

		
	return True
		
	
	
#
#
#	G E T   C O U N T
#
# just returns the count of the images on the page so we know how many airports we need
# to talk to in order to get the needed info.
#
def getCount():
	if XTension.debugMode:
		XTension.writeLog( "begin getCount")
		
	try:
		reply = subprocess.check_output( ["osascript", scriptsFolder + "get_count.scpt", "-s", "o"])
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no message"
		
		XTension.writeLog( "Error getting count of airports (" + str( e.returncode) + ") " + theMessage + " " + reply, xtLogRed)
		XTension.setRunState( xtRunStateError, "error getting count of airports (" + str( e.returncode) + ") "  + theMessage + " " + reply)
		return False
		
	reply = reply.strip()
		
	# if we get the timeout message then also return an error
	if reply == 'timeout':
		XTension.writeLog( "Timeout while getting internet status", xtLogRed)
		XTension.setRunState( xtRunStateError, "timeout while getting internet status")
		return False

	global airportCount
	airportCount = int( reply)
	
	if XTension.debugMode:
		XTension.writeLog( str( airportCount) + " airports found")
		
	return True



	
#
#	G E T   S T A T U S
#
#  gets the status of every airport
#

def getStatus():

	#
	# clear the global error flag
	# if this is still false at the end of this
	# method we will make sure that the period of the
	# scanTimer is as the preferences have set and 
	# if it's true then we will set the period to 1 minute
	# until the airport comes back online
	#
	global airportHasError
	airportHasError = False

	#
	# manage structures for client handling
	#
	clientsInXTension = []	
	# get the flag for if we should do this or not from the preferences
	sendClients = XTension.settings.get( keyScanClients, False)

	# get a list of client units from XTension so that we can tell if some need to be
	# turned off as they won't be in this list.
	if sendClients:
		clientsInXTension = getAllClientUnits()




	# 
	# walk all the images and get their information
	#


	for imageIndex in range( 2, airportCount + 1):
		if XTension.debugMode:
			XTension.writeLog( "begin get status for index " + str( imageIndex))
			
		try:
			reply = subprocess.check_output( ["osascript", "-s", "so", scriptsFolder + "get_airport_status.scpt", str( imageIndex)])
			reply = reply.strip() # always strip any osascript reply as it adds a newline at the end of it and nothing will match otherwise
			#XTension.writeLog( "the reply =" + reply)
		except subprocess.CalledProcessError, e:
			theMessage = e.output
			if theMessage == None:
				theMessage = "no message"
			
			XTension.writeLog( "Error getting status for airport at index " + str( imageIndex - 1), xtLogRed)
			XTension.setRunState( xtRunStateError, "error getting status for airport at index " + str( imageIndex - 1))
			continue
			
		if reply == "timeout":
			XTension.writeLog( "Timeout while getting airport status at index: " + str( imageIndex - 1), xtLogRed)
			XTension.setRunState( xtRunStateError, "timeout while getting airport status at index: " + str( imageIndex - 1))
			continue
			
		#XTension.writeLog( reply)
		
		# at this point we've got an applescript structure list like {"something", "something"}
		# step one is to strip the first and last 2 chars off the string
		
		reply = reply[2: -2]
		elements = reply.split( "\", \"")
		
		#for thisElement in elements:
		#	XTension.writeLog( thisElement)
			
		thisName = elements[0]
		thisAddress = elements[1]
		thisStatusText = elements[2]
		
		#
		# if the airport is down then it might return an error here
		#
		
		if thisName == "error":
			statusUnit = getAirportAtIndex( imageIndex)
			if statusUnit == None:
				XTension.writeLog( "unable to find the airport at index " + str( imageIndex) + ", cannot process that entry.", xtLogRed)
				continue # but keep scanning
			# if we found the unit then conform the name so we can continue to say that it's down
			thisName = statusUnit.data.get( keyAirportAtIndex)
			
		
		#XTension.writeLog( "name=" + thisName + " address=" + thisAddress + " status=" + thisStatusText)
		
		# if it ends with working normally then all is well
		thisStatus = thisStatusText.endswith( keyWorkingNormally)
		
		# this will set the period of the scan timer to one minute at the end of this method
		if thisStatus == False:
			airportHasError = True
		
		if XTension.debugMode:
			if thisStatus:
				XTension.writeLog( "status is good")
			else:
				XTension.writeLog( "status is bad")
		
		
		# pull out any clients from the list in case we're going to process them into units
		clients = elements[ 3:]
		
		# if it's a new unit and at index 2 then we will set this
		# so that the code below knows that we should treat this one as the router and
		# process it's IP address as the external IP address. Setting the value in the
		# XTension data via the command below won't have echo'ed back to us yet for this
		# so we need a local way to know this was set in that case.
		localRouterOverride = False
		
		#
		# if the status of the unit is already on or off then we don't want to send another
		# one to it. If it's a new unit or if the current status is different then we do
		#
		#sendStatusUpdate = False
		
		workUnit = XTension.getUnitFromAddress( tagAirportStatus, thisName.upper())
		if workUnit == None:
			#XTension.writeLog( "no work unit found, creating it")
			#sendStatusUpdate = True
			
			# create the unit
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandNewUnit)
			x.set( xtKeyTag, tagAirportStatus)
			x.set( xtKeyAddress, thisName.upper())
			x.set( xtKeyReadOnly, xtTrue)
			x.set( xtKeyName, "Airport: " + thisName)
			x.set( xtKeyList, listForNewUnits)
			
			XTension.sendCommand( x)
			
			# also if the unit was not there and we are at index 2 then we should set the default
			# value of isRouter to true for this unit.
			
			if imageIndex == 2:
				localRouterOverride = True
				
				x = XTCommand()
				x.set( xtKeyCommand, xtCommandSetXMLData)
				x.set( xtKeyAddress, thisName.upper())
				x.set( xtKeyTag, tagAirportStatus)
				x.set( xtKeyName, keyIsRouter)
				x.set( xtKeyValue, xtTrue)
				
				XTension.sendCommand( x)
		#else:
			# there was a valid unit already in XTension
			# don't bother to send the command here unless the
			# value is different than the current value
			
			#currentStatus = (int( workUnit.data.get( xtKeyValue)) != 0)
			#sendStatusUpdate = (currentStatus != thisStatus)
			# actually don't bother with this check, just send the command
			
	
		#XTension.writeLog( "unit was found, sending it the command")
		x = XTCommand()
		x.set( xtKeyTag, tagAirportStatus)
		x.set( xtKeyAddress, thisName.upper())
		x.set( xtKeyUpdateOnly, xtTrue)

		if thisStatus:
			x.set( xtKeyCommand, xtCommandOn)
			x.set( xtKeyDefaultLabel, "Up")
		else:
			x.set( xtKeyCommand, xtCommandOff)
			x.set( xtKeyDefaultLabel, "Down")

		# if the address is an error then none of the rest of this is valid either
		# so don't send it
	
		if thisAddress != "error":			
			x.set( xtKeyUnitPropertyName + "0", "client count")
			x.set( xtKeyUnitPropertyValue + "0", str( len( clients)))

			x.set( xtKeyUnitPropertyName + "1", "address")
			x.set( xtKeyUnitPropertyValue + "1", thisAddress)
	
	
		XTension.sendCommand( x)
		

		
		#
		# now set the description to the entire text of this but only if it's different from
		# the current description of the unit which we have in the workUnit XTData
		# 
		
		sendDescription = True
		saveIndexToData = True
		
		
		if workUnit != None:
			saveIndexToData = (workUnit.data.get( keyAirportAtIndex) != str( imageIndex)) # we will send this if it's not what it should be
			
			if workUnit.data.get( xtKeyDescription) == thisStatusText:
				sendDescription = False
							
			if workUnit.data.get( keyIsRouter, False) == True:			
				workAddressUnit = XTension.getUnitFromAddress( tagInternetStatus, addressIp)
				
				if workAddressUnit == None or workAddressUnit.data.get( xtKeyDescription) != thisAddress:
					localRouterOverride = True
				
				
		

		# send the IP address if this is the IP router			
		if localRouterOverride == True and thisAddress != "error":
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandSetDescription)
			x.set( xtKeyAddress, addressIp)
			x.set( xtKeyTag, tagInternetStatus)
			x.set( xtKeyData, thisAddress)
			
			XTension.sendCommand( x)
			
			# and now execute the special script handler for the new IP address received
			
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandScriptHandler)
			x.set( xtKeyAddress, addressIp)
			x.set( xtKeyTag, tagInternetStatus)
			x.set( xtKeyName, IPChangedScriptHandler)
			x.set( xtKeyData, thisAddress)
			
			XTension.sendCommand( x)
					
				

			

		
		if sendDescription:
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandSetDescription)
			x.set( xtKeyAddress, thisName.upper())
			x.set( xtKeyTag, tagAirportStatus)
			x.set( xtKeyData, thisStatusText)
			XTension.sendCommand( x)
			
			# also write to the log if the description has changed
			XTension.writeLog( thisStatusText)
			
		# save off the index into the unit so that we can 
		# compare it later if we get an error
		
		if saveIndexToData:
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandSetXMLData)
			x.set( xtKeyAddress, thisName.upper())
			x.set( xtKeyTag, tagAirportStatus)
			x.set( xtKeyName, keyAirportAtIndex)
			x.set( xtKeyValue, str( imageIndex))
			
			#XTension.writeLog( "saving the index")
			#x._debugLog()
			
			XTension.sendCommand( x)
	
		
		#XTension.writeLog( "done with sending command")	
		
		
		
		#
		#  if send clients is turned on then manage that system too
		#
		if sendClients:
			#XTension.writeLog( "in send clients handling")
			#print( clients)
			# get a list of client units from XTension so that we can tell if some need to be
			# turned off as they won't be in this list.
			
			#	def containersWithValue( self, containerKey, keyToCompare, value):
			#clientsInXTension = getAllClientUnits()
			
			for foundClientAddress in clients:
				nameAsAddress = filterNameToAddress( foundClientAddress) # all addresses in XTension are uppercase
				# sometimes the addresses are returned with .local at the end and sometimes not resulting
				# in multiple units, so pull off the .local if it's there
				
				
				if nameAsAddress in clientsInXTension:
					#XTension.writeLog( "client address (" + nameAsAddress + ") was found, removing it")
					clientsInXTension.remove( nameAsAddress)
				#else:
				#	XTension.writeLog( "client address (" + nameAsAddress + ") was NOT found in the list")

				
				
				# if we found it then the unit should be in XTension and it should be On
				# address should have another address segment that is the name of the airport so that
				# we don't keep turning things on and off as you walk from one airport to another
				#
				workUnit = XTension.getUnitFromAddress( tagAirportClient, nameAsAddress)
				if workUnit == None:
					# it's not in XTension, so create a unit for it
					#XTension.writeLog( "creating a unit for address (" + nameAsAddress + ")", xtLogGreen)
					
					x = XTCommand()
					x.set( xtKeyCommand, xtCommandNewUnit)
					x.set( xtKeyTag, tagAirportClient)
					x.set( xtKeyAddress, nameAsAddress)
					x.set( xtKeyName, "Airport Client: " + foundClientAddress)
					x.set( xtKeyReadOnly, xtTrue)
					x.set( xtKeyList, listForNewUnits)
					x.set( xtKeyOnLabel, "OnLine")
					x.set( xtKeyOffLabel, "OffLine")
					x.set( xtKeyValue, "1") # it is present if it is here
					
					XTension.sendCommand( x)
					
				else:
					# the unit was found in XTension
					# if the state in XTension is not on then send an On for it
					# then delete the name from the clientsInXTension list
					# so we can send Off's for any units left in it that are no longer listed
					#XTension.writeLog( "found unit at address (" + nameAsAddress + ")", xtLogGreen)
					if int( workUnit.data.get( xtUnitKeyValue)) < 1:
						#XTension.writeLog( "turning it on as it's not on")
						x = XTCommand()
						x.set( xtKeyCommand, xtCommandOn)
						x.set( xtKeyTag, tagAirportClient)
						x.set( xtKeyAddress, nameAsAddress)
						
						XTension.sendCommand( x)
					#else:
						#XTension.writeLog( "it seems to be already on")
						
						
				
					
								
			
	# at this point we are done looping through the 
	# images and the clientsInXTension should hold the names of any 
	# that were not found anywhere on any of the airports so we should send an off for any
	# of them that are still in the list.
	
	for workUnitAddress in clientsInXTension:
		# only send this if the client unit isn't already off
		
		clientUnit = XTension.getUnitFromAddress( tagAirportClient, workUnitAddress)
		
		if clientUnit == None or clientUnit.data.get( xtUnitKeyValue) > 0:
		
			x = XTCommand()
			x.set( xtKeyCommand, xtCommandOff)
			x.set( xtKeyTag, tagAirportClient)
			x.set( xtKeyAddress, workUnitAddress.upper())
			#XTension.writeLog( "turning off (" + workUnitAddress.upper() + ")")
		
			XTension.sendCommand( x)
				
		
	return True
	
#
#	G E T   A L L   C L I E N T   U N I T S
#
# used by the above getStatus call to get all the client units from the XTension
# database. They are removed one at a time from the list of current units
# so that we can see if any need to be turned off
#
def getAllClientUnits():

	foundClients = []
	
	for thisId in XTension.unitIndexById:
		thisUnit = XTension.unitIndexById[ thisId]
		if thisUnit.data.get( xtUnitKeyTag) == tagAirportClient:
			foundClients.append( thisUnit.address)
			#XTension.writeLog( "found existing unit at address: (" + thisUnit.address + ")", xtLogGreen)
			
	
	return foundClients
	
#
#	G E T   A I R P O R T   A T   I N D E X
#
# used if an error happens as we may not be able to get the name from the popup as it won't popup
# so whenever we scan an airport we save the index at which we found it so we can find it by just the index
#

def getAirportAtIndex( theIndex):

	for thisId in XTension.unitIndexById:
		thisUnit = XTension.unitIndexById[ thisId]
		if thisUnit.data.get( xtUnitKeyTag) == tagAirportStatus and thisUnit.data.get( keyAirportAtIndex) == str( theIndex):
			return thisUnit
		
	return None


#
#	Q U I T   A I R P O R T
#
def quitAirportUtility():
	if XTension.debugMode:
		XTension.writeLog( "begin quitAirportUtility")
		
		
	try:
		reply = subprocess.check_output( ["osascript", "-s", "o", scriptsFolder + "quit_airport_utility.scpt"])
		reply = reply.strip()
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no message"
	
		XTension.writeLog( "Error quitting airport utility (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		return False
		
	if reply == "timeout":
		XTension.writeLog( "timeout while trying to quit airport utility", xtLogRed)
		XTension.setRunState( xtRunStateError, "timeout quitting airport utility")
		return False
		
	return True

		
	


#
#	V E R I F Y   A I R P O R T   U T I L I T Y
#
# makes sure that the airport utility is running
# and if not launches it, will perform a delay if the
# utility was not running
#
# the script will return a string of "true" if it's running 
# and "false" if it is not.
#
# MUST BE CALLED ONLY FROM A THREAD 
#
def verifyAirportUtility():
	if XTension.debugMode:
		XTension.writeLog( "begin verifyAirportUtility")
		
	try:
		reply = subprocess.check_output( ["osascript", scriptsFolder + "start_airport_utility.scpt", "-s", "o"])
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no message"
	
		XTension.writeLog( "Error starting airport utility (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		return False
		
	reply = reply.strip()
		
	if reply == "timeout":
		XTension.writeLog( "timeout while trying to launch airport utility", xtLogRed)
		XTension.setRunState( xtRunStateError, "timeout launching airport utility")
		return False

	if reply == "false":
		if XTension.debugMode:
			XTension.writeLog( "airport utilty was not running, waiting for it to launch")
		#it was not running, sleep a moment to make sure it's actually ready before we continue
		sleep( 7)

	if XTension.debugMode:
		XTension.writeLog( "end verifyAirportUtility")
		
	return True


#
#	S C A N   T I M E R
#

class xtScanTimer( xtTimer):
	def __init__( self, mperiod):
		xtTimer.__init__( self, mperiod)
		
	def action( self):		
		startScan()
		


#
#  G O T   S E T T I N G S
#
# called when the settings have been saved in XTension, or also when we are first launched
# we are looking for a change in the scanInterval settings so we can reset our timer

def gotSettings():
	global scanInterval
	newInterval = int( XTension.settings.get( keyScanFrequency, "5"))
	if newInterval != scanInterval:
	
		if newInterval < 1:
			XTension.writeLog( "scan interval of " + str( newInterval) + " cannot be less than 1. No changes made", xtLogRed)
			return
	
		
		scanInterval = newInterval
		
		if scanTimer != None and scanTimer.period != scanInterval:
		
			if XTension.debugMode:
				XTension.writeLog( "scan interval changed to: " + str( scanInterval) + " minutes")
				
			scanTimer.setPeriod( scanInterval)
			
	
#
#	S C A N   N O W
#
# called when you click scan now in the UI or use the scanNow() command
#

def scanNow( commandName, positionalParms, dataParms):
	XTension.writeLog( "Beginning Airport Scan")
	startScan()
	
#
#	R E B O O T
#
# the script command to actually do the reboot
# this just starts the thread to do it as we can't 
# hang up the receiving thread that long or XTension might restart us
#

def rebootMenu( commandName, positionalParms, dataParms):

		#dataParms._debugLog()
		
		if dataParms.get( xtKeyTag) != tagAirportStatus:
			XTension.writeLog( "unable to restart a non Airport Status unit type", xtLogRed)
			return
		
		thisAddress = dataParms.get( xtKeyAddress)
		# but the address no longer equates to the actual name in the airport
		# so we have to pull the index from the unit.
		
		workUnit = XTension.getUnitFromAddress( tagAirportStatus, thisAddress)
		
		if workUnit == None:
			XTension.writeLog( "unable to find a unit representing an airport with address (" + thisAddress + ") no reboot attempted.", xtLogRed)
			return
		
		# make sure we have an index stored for this unit
		
		if not workUnit.data.exists( keyAirportAtIndex):
			XTension.writeLog( "no index stored for airport with address (" + thisAddress + ") no reboot can be attempted.", xtLogRed)
			XTension.writeLog( "make sure to complete at least one scan with the airport visible before attempting to restart it", xtLogRed)
			return
			
		thisIndex = str( workUnit.data.get( keyAirportAtIndex))
		
		XTension.writeLog( "beginning reboot for airport at address (" + thisAddress + ") index: " + thisIndex)
				
		rebootThread = Thread( target=threadedReboot, args=( thisIndex,))
		rebootThread.start()


#
# 	T H R E A D E D   R E B O O T
#
# started as a thread from the rebootMenu handler. Is passed the index of the image
# that we want to click to do a restart. Since we can no longer use the address of the unit
# to directly link to an airport by name we just use the index that is saved off whenever the
# airports are scanned.
#

def threadedReboot( theIndex):

	if not controlLock.acquire( False):
		XTension.writeLog( "control thread is already busy, blocking the reboot thread and waiting for it to complete", xtLogRed)
		controlLock.acquire()

	if not verifyAirportUtility():
		sleep( 5)
		

	try:
		reply = subprocess.check_output( ["osascript", "-s", "o", scriptsFolder + "reboot.scpt", theIndex])
		reply = reply.strip()
		#
		# if anything other than "true" was returned then thats an error and we should log it
		# and probably execute an error handler as well so that you know that it failed...
		#
		if reply != "true":
			XTension.writeLog( "reboot failed for index " + str( theIndex) + " with message: " + reply, xtLogRed)
		else:
			XTension.writeLog( "reboot command sent")
			# now do a scan in a minute to see if things come back up
			# if any airport is down then the period is set to one minute until it comes back online anyway
			if scanTimer.period != 30:
				scanTimer.setPeriod( 30)
		
	except subprocess.CalledProcessError, e:
		theMessage = e.output
		if theMessage == None:
			theMessage = "no message"
		
		XTension.writeLog( "Error while rebooting (" + str( e.returncode) + ") " + theMessage, xtLogRed)
		XTension.setRunState( xtRunStateError, "error while rebooting (" + str( e.returncode) + ") "  + theMessage )

		
	controlLock.release()
	return
		
	



#
#	M A I N
#


XTension = cXTension()
#XTension.debugMode 		= True
XTension.onGotSettings = gotSettings # so we can update the period of our timer
XTension.addScriptHandler( keyScanNowScript, scanNow)
XTension.addScriptHandler( keyRebootMenu, rebootMenu)
XTension.addScriptHandler( keyRebootScript, rebootMenu)
XTension.startup()





if not XTension.ready:
	print( "Failed to make connection to XTension. Shutting down\r")
	sys.exit()
else:
	XTension.writeLog( "XTension AirportInfo plugin v" + pluginVersion + " starting up...")


# sleep for a moment to let a startup settle down and then start a scan and also setup
# a timer object to do it every few minutes

sleep( 2)

startScan()



scanTimer = xtScanTimer( 60 * scanInterval)
scanTimer.start()
